@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Annotation.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -168,9 +168,9 @@ sub annotate_with_glimmer {
 sub annotate_with_LORF {
     my $this = shift;
     my $gb = shift;
+    my $min = shift || 50;
+    my $max = shift || 200000;
     my $seq = $gb->{SEQ};
-    my ($start,$end,$i);
-    my $count = 0;
 
     $this->{"CDS0"}->{dummy} = 1;
     for my $key (qw/LOCUS HEADER COMMENT FEATURE0 SEQ/){
@@ -179,45 +179,46 @@ sub annotate_with_LORF {
 
     $this->{FEATURE0}->{on} = 1 unless(defined $this->{FEATURE0});
 
-    for ($i = 0; $i <= 1; $i ++){
-	$seq = complement($gb->{SEQ}) if ($i);
-	$start = 0;
-	$end = 0;
-	
-	while(0 <= ($start = index($seq, 'atg', $start + 1))){
-	    next if ($start < $end && ($start - $end + 1) % 3 == 0);
+    my $i     = 0;
+    my $count = 0;
 
+    for ($i = 0; $i <= 1; $i ++){
+	$seq      = complement($gb->{SEQ}) if ($i);
+	my $start = 0;
+	my $end   = 0;
+
+	my %stop;
+	while(0 <= ($start = index($seq, 'atg', $start + 1))){
 	    for my $codon (qw/tag taa tga/){
 		my $tmp = $start;
-		my $flag = 0;
-
-		while(0 <= ($tmp = index($seq, $codon, $tmp +1))){
-		    if (($tmp - $start + 1) % 3 == 0 && $tmp - $start > 49){
-			$count ++;
-			$end = $tmp;
-			$flag = 1;
-
-			if ($i){
-			    $this->{"FEATURE$count"}->{start} = length($gb->{SEQ}) - $end + 1;
-			    $this->{"FEATURE$count"}->{end} = length($gb->{SEQ}) - $start + 1;
-			    $this->{"FEATURE$count"}->{direction} = "complement";
-			}else{
-			    $this->{"FEATURE$count"}->{start} = $start + 1;
-			    $this->{"FEATURE$count"}->{end} = $end + 1;
-			    $this->{"FEATURE$count"}->{direction} = "direct";
+		while(0 <= ($tmp = index($seq, $codon, $tmp + 1))){
+		    if (($tmp - $start + 1) % 3 == 0 && $tmp - $start >= $min && $tmp - $start <= $max){
+			if($tmp < $end || $end == 0){
+			    $end = $tmp;
 			}
-			
-			$this->{"FEATURE$count"}->{type} = "CDS";
-			$this->{"FEATURE$count"}->{gene} = "FEATURE$count";
-			$this->{"FEATURE$count"}->{feature} = $count;
-			$this->{"FEATURE$count"}->{cds} = $count;
-			$this->{"FEATURE$count"}->{on} = 1;
-			$this->{"CDS$count"} = $this->{"FEATURE$count"};
-			
-			last;
 		    }
 		}
-		last if ($flag);
+	    }
+	    if($end != 0 && $stop{$end} < 1){
+		$count ++;
+		$stop{$end} ++;
+		
+		if ($i){
+		    $this->{"FEATURE$count"}->{start}     = length($gb->{SEQ}) - $end   + 1;
+		    $this->{"FEATURE$count"}->{end}       = length($gb->{SEQ}) - $start + 1;
+		    $this->{"FEATURE$count"}->{direction} = "complement";
+		}else{
+		    $this->{"FEATURE$count"}->{start}     = $start + 1;
+		    $this->{"FEATURE$count"}->{end}       = $end   + 1;
+		    $this->{"FEATURE$count"}->{direction} = 'direct';
+		}
+		
+		$this->{"FEATURE$count"}->{type}    = 'CDS';
+		$this->{"FEATURE$count"}->{gene}    = "FEATURE$count";
+		$this->{"FEATURE$count"}->{feature} = $count;
+		$this->{"FEATURE$count"}->{cds}     = $count;
+		$this->{"FEATURE$count"}->{on}      = 1;
+		$this->{"CDS$count"}                = $this->{"FEATURE$count"};
 	    }
 	}
     }
