@@ -144,6 +144,10 @@ sub read_header{
     $this->{TAXONOMY}->{0} =~ s/\n//g;
     $this->{TAXONOMY}->{0} =~ s/\s+ / /g;
     $this->{TAXONOMY}->{0} =~ s/; /;/g;
+    $this->{TAXONOMY}->{0} =~ s/Bacillales/Bacilli;Bacillales/;
+    $this->{TAXONOMY}->{0} =~ s/Lactobacillales/Bacilli;Lactobacillales/;
+    $this->{TAXONOMY}->{0} =~ s/Staphylococcus/Staphylococcaceae;Staphylococcus/;
+    $this->{TAXONOMY}->{0} =~ s/Synechococcus/Synechococcales;Synechococcaceae;Synechococcus/;
 
     my $i = 1;
     foreach my $entry (split(/;/, $this->{TAXONOMY}->{0})){
