@@ -112,7 +112,11 @@ sub make_gb {
 			
             }else{
                 my @partial = split(/ /,$gb->{$feat}->{"partial"});
-                if($partial[1] == 1){
+		if($partial[1] == 1 && $partial[0] == 1){
+                    printf OUT "%-4.4s %-15.15s %s\n",
+			    "$dummy","$gb->{$feat}->{type}",
+			    "<$gb->{$feat}->{start}..>$gb->{$feat}->{end}";
+		}elsif($partial[1] == 1){
                     printf OUT "%-4.4s %-15.15s %s\n",
 			    "$dummy","$gb->{$feat}->{type}",
 			    "$gb->{$feat}->{start}..>$gb->{$feat}->{end}";
@@ -162,9 +166,24 @@ sub make_gb {
 		}
 
 	    }else{
-                printf OUT "%-4.4s %-15.15s %s\n",
-			"$dummy","$gb->{$feat}->{type}",
-			"$gb->{$feat}->{direction}($gb->{$feat}->{start}..$gb->{$feat}->{end})";
+                my @partial = split(/ /,$gb->{$feat}->{"partial"});
+		if($partial[1] == 1 && $partial[0] == 1){
+                    printf OUT "%-4.4s %-15.15s %s\n",
+			    "$dummy","$gb->{$feat}->{type}",
+			    "complement(<$gb->{$feat}->{start}..>$gb->{$feat}->{end})";
+		}elsif($partial[1] == 1){
+                    printf OUT "%-4.4s %-15.15s %s\n",
+			    "$dummy","$gb->{$feat}->{type}",
+			    "complement($gb->{$feat}->{start}..>$gb->{$feat}->{end})";
+                }elsif($partial[0] == 1){
+                    printf OUT "%-4.4s %-15.15s %s\n",
+			    "$dummy","$gb->{$feat}->{type}",
+			    "complement(<$gb->{$feat}->{start}..$gb->{$feat}->{end})";
+                }else{
+                    printf OUT "%-4.4s %-15.15s %s\n","$dummy",
+			    "$gb->{$feat}->{type}",
+			    "complement($gb->{$feat}->{start}..$gb->{$feat}->{end})";
+		}
             }
 	}
 
