@@ -900,7 +900,7 @@ sub get_intron {
 
 =head2 $gb->get_exon()
 
-   Name: $gb->get_exon()   -   get a list of exon sequences of the given CDS
+   Name: $gb->get_exon()   -   get a concatenated exon sequence of the given CDS
 
    Description:
          Given a CDS ID, returns the exon sequence.
@@ -938,6 +938,50 @@ sub get_exon {
 }
 
 
+=head2 $gb->get_exons()
+
+   Name: $gb->get_exons()   -   get a list of exon sequences of the given CDS
+
+   Description:
+         Given a CDS ID, returns the sequences of exon sequence.
+         'complement' is properly parsed, and introns are spliced out.
+
+         eg. $gb->get_exons('CDS1'); returns the array of exon sequences.
+
+   REST: 
+      http://rest.g-language.org/NG_016275/get_exons/IL10RA
+
+=cut
+
+sub get_exons {
+    if (scalar(@_) < 2){
+	carp("Not enough arguments.");
+	return;
+    }
+    my $this = shift;
+    my $cds = shift;
+
+    return unless (length $this->{$cds}->{join});
+
+    my @seq;
+
+    foreach my $line (split(/,/, $this->{$cds}->{join})){
+	my $complement = $line =~ tr/c//d;
+	my ($start, $end) = split(/\.\./, $line, 2);
+	my $tmp = $this->get_gbkseq($start, $end);
+	$tmp = complement($tmp) if ($complement);
+	
+	if($this->{$cds}->{direction} eq 'complement'){
+	    unshift(@seq, complement($tmp));
+	}else{
+	    push(@seq, $tmp);
+	}
+    }
+
+    return @seq;
+}
+
+
 
 =head2 $gb->seq_info()
 
