@@ -117,6 +117,9 @@ sub say{
 
     -format      'plain' to read as plain text, 'url' to read as url,
                  'fasta' for FASTA file, 'fastq' for FASTQ file.
+                 'kegg' for parsing KEGG flatfiles, and 'swiss' for 
+                 parsing UniProt flatfiles, returning a data structure
+                 object representing the file content. 
                  Usually the format is automatically interpreted, but
                  this option can be specified to force interpretation.
     -gzip        0 for uncompressed files, 1 for gzipped files (default:0)
@@ -126,6 +129,7 @@ sub say{
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20100310-01 added support for kegg/swissprot parsing
     20100122-01 added support for gzipped files
     20091202-01 added FastA/FastQ support
     20090520-01 added URL support
@@ -141,6 +145,48 @@ sub readFile{
     my $format  = lc(opt_val('format'));
     my $gzip    = lc(opt_val('gzip'));
 
+    if($format eq "kegg"){
+	my ($kegg, $key, $entry);
+
+	open(FILE, $file) || croak($!);
+	while(<FILE>){
+	    chomp;
+	    if(/^(\S+?)\s+(.*)/){
+		my $two = $2;
+		if ($1 eq 'ENTRY'){
+		    ($entry, undef) = split(/\s+/, $two);
+		}else{
+		    $key = $1;
+		    $kegg->{$entry}->{$key} = $two;
+		}
+	    }elsif(/^\s+(.*)/){
+		$kegg->{$entry}->{$key} .= ' ' . $1;
+	    }
+	}
+	close(FILE);
+	
+	return $kegg;
+    }elsif($format =~ /swiss/){
+	my $sprot = {};
+	my $key = '';
+
+	open(FILE, $file) || croak($!);
+	while(<FILE>){
+	    chomp;
+	    if (/^ID\s+(.+?)\s+/){
+		$key = $1;
+		$sprot->{$key}->{LOCUS}->{id} = $key;
+	    }elsif(/^\/\//){
+		$key = '';
+	    }elsif(/^(..)\s+(.*)/){
+                $sprot->{$key}->{$1} .= $2;
+	    }
+	}
+	close(FILE);
+	
+	return $sprot;
+    }
+
     unless($format){
 	if (lc($file) =~ /\.(fasta|fast|seq|fst|fa|fsa|nt|aa|ffn|fna)/i){
 	    $format = 'fasta';
