@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Align.pm,v 1.2 2002/07/30 17:40:56 gaou Exp $
@@ -60,6 +60,30 @@ our @EXPORT = qw(
 #       Let the code begin...
 #:::::::::::::::::::::::::::::::::
 
+
+=head2 G::Seq::AminoAcid::monoisotopic
+
+  Name: G::Seq::AminoAcid::monoisotopic   -   returns the monoisotopic mass of a given amino acid
+
+  Description:
+    This method returns the monoisotopic mass of a given amino acid.
+
+  Usage: 
+    $mass = G::Seq::AminoAcid::monoisotopic($aminocid);
+    eg. G::Seq::AminoAcid::monoisotopic('A'); # returns 71.04;
+
+  Options:
+    none
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
+
+=cut
+
+
 sub monoisotopic{
     my $key = shift;
 
@@ -74,6 +98,28 @@ sub monoisotopic{
     return %monoisotopic;
 }
 
+=head2 G::Seq::AminoAcid::charge
+
+  Name: G::Seq::AminoAcid::charge   -   returns the charge of a given amino acid
+
+  Description:
+    This method returns the charge of a given amino acid.
+
+  Usage: 
+    $charge = G::Seq::AminoAcid::charge($aminocid);
+    eg. G::Seq::AminoAcid::charge('A'); # returns 0
+
+  Options:
+    none
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
+
+=cut
+
 sub charge {
     my $key = shift;
 
@@ -117,6 +163,28 @@ sub dayhoff {
     return %dayhoff;
 }
 
+=head2 G::Seq::AminoAcid::residue
+
+  Name: G::Seq::AminoAcid::residue   -   retunrs the mass of a given amino acid residue
+
+  Description:
+    This method returns the mass of a given amino acid residue.
+
+  Usage: 
+    $mass = G::Seq::AminoAcid::residue($aminocid);
+    eg. G::Seq::AminoAcid::residue('A'); # returns 71.0786
+
+  Options:
+    none
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
+
+=cut
+
 sub residue{
     my $key = shift;
 
@@ -131,6 +199,28 @@ sub residue{
     return %residue;
 }
 
+=head2 G::Seq::AminoAcid::mass
+
+  Name: G::Seq::AminoAcid::mass   -   retunrs the mass of a given amino acid
+
+  Description:
+    This method returns the mass of a given amino acid.
+
+  Usage: 
+    $mass = G::Seq::AminoAcid::mass($aminocid);
+    eg. G::Seq::AminoAcid::mass('A'); # returns 89.09
+
+  Options:
+    none
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
+
+=cut
+
 sub mass{
     my $key = shift;
 
@@ -145,10 +235,35 @@ sub mass{
     return %mass;
 }
 
+=head2 G::Seq::AminoAcid::hydropathy
+
+  Name: G::Seq::AminoAcid::hydropathy   -   returns the hydropathy of given amino acid
+
+  Description:
+    This method returns the hydropathy of a given amino acid.
+
+  Usage: 
+    $hydropathy = G::Seq::AminoAcid::hydropathy($aminocid);
+    eg. G::Seq::AminoAcid::hydropathy('A'); # returns 1.8;
+
+  Options:
+    none
+
+  Author: 
+    Haruo Suzuki (hs568@cornell.edu)
+
+  References:
+   1. Kyte J, Doolittle RF (1982) "A simple method for displaying the 
+      hydropathic character of a protein", J Mol Biol, 157(1):105-32. 
+
+  History:
+    20091204-01 initial posting
+
+=cut
+
 sub hydropathy{
     my $key = shift;
 
-    # Kyte J, Doolittle RF. A simple method for displaying the hydropathic character of a protein. J Mol Biol. 1982 157(1):105-32. 
     my %hydropathy = (
 		      A=> 1.8, C=> 2.5, D=>-3.5, E=>-3.5, F=> 2.8,
 		      G=>-0.4, H=>-3.2, I=> 4.5, K=>-3.9, L=> 3.8,
@@ -160,6 +275,28 @@ sub hydropathy{
     return %hydropathy;
 }
 
+=head2 G::Seq::AminoAcid::one2three
+
+  Name: G::Seq::AminoAcid::one2three   -   converts one letter amino acid code to three letter code
+
+  Description:
+    This method converts one letter amino acid code to three letter code.
+
+  Usage: 
+    $amino = G::Seq::AminoAcid::one2three($aminocid);
+    eg. G::Seq::AminoAcid::one2three('A'); # returns 'Ala';
+
+  Options:
+    none
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
+
+=cut
+
 sub one2three{
     my $key = shift;
 
@@ -175,6 +312,28 @@ sub one2three{
     return %one2three;
 }
 
+=head2 G::Seq::AminoAcid::three2one
+
+  Name: G::Seq::AminoAcid::three2one   -   converts three letter amino acid code to one letter code
+
+  Description:
+    This method converts three letter amino acid code to one letter code.
+
+  Usage: 
+    $amino = G::Seq::AminoAcid::three2one($aminocid);
+    eg. G::Seq::AminoAcid::three2one('Ala'); # returns 'A';
+
+ Options:
+    none
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
+
+=cut
+
 sub three2one{
     my $key = shift;
     my %three2one = reverse one2three();
@@ -185,11 +344,22 @@ sub three2one{
 
 =head2 peptide_mass
 
-       Author: Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
-        Usage: double $mass = peptide_mass(string $aminoseq);
-      Options: None.
-  Description: Calculates mass of given amino acid sequence.
-      History: created 2001.04.04
+  Name: peptide_mass   -   calculates mass of given amino acid sequence
+
+  Description:
+    This method calculates the calculates mass of given amino acid sequence.
+
+  Usage: 
+    $mass = peptide_mass($aminoseq);
+
+ Options:
+    none
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
 
 =cut
 
@@ -208,11 +378,22 @@ sub peptide_mass{
 
 =head2 calc_pI
 
-       Author: Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
-        Usage: double $pI = calc_pI(string $aminoseq);
-      Options: None.
-  Description: Calculates isoelectric point of given amino acid sequence.
-      History: created 2001.04.04
+  Name: calc_pI   -   calculates the isoelectric point of given amino acid sequence
+
+  Description:
+    This method calculates the isoelectric point of given amino acid sequence.
+
+  Usage: 
+    $pI = calc_pI($aminoseq);
+
+ Options:
+    none
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
 
 =cut
 
@@ -268,15 +449,34 @@ sub calc_pI{
 
 
     
+=head2 amino_info
 
-=head2 _charge
+  Name: amino_info   -   prints out basic amino acid sequence statistics
 
-       Author: Misa Nakanishi (dora@g-language.org)
-        Usage: amino_info(string $aminoseq)
-      Options: None.
-  Description: Prints basic amino acid sequence information
-      History: created 2001.04.04 
-               Tuned by Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+  Description:
+    This method prints out basic compositional statistics of the 
+    given amino acid sequence in a human readable manner.
+
+    Displayed information includes the following:
+      Molecular weight, number of residues, average residue weight, charge,
+      isoelectric point, number/mole/DayhoffStat of each amino acid, and
+      percentage of Tiny (A+C+G+S+T), Small (A+B+C+D+G+N+P+S+T+V), 
+      Aliphatic (I+L+V), Aromatic (F+H+W+Y), Non-polar (A+C+F+G+I+L+M+P+V+W+Y), 
+      Polar (D+E+H+K+N+Q+R+S+T+Z), Charged (B+D+E+H+K+R+Z), Basic (H+K+R),
+      Acidic (B+D+E+Z) residues.
+
+  Usage: 
+    null = amino_info($sequence);
+
+ Options:
+    none
+
+  Author: 
+    Misa Nakanishi (dora@g-language.org)
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010404-01 initial posting
 
 =cut
 
