@@ -143,7 +143,7 @@ sub codon_compiler {
     elsif($data eq 'RSCU'){ $data = 'R3'; }
     elsif($data eq 'W'){ $data = 'R4'; }
     if($data !~ /A[01]|[CR][0-5]/){ &msg_error("$data: undefined type of amino acid and codon usage.\n"); return; }
-    if($data =~ /A[01]/){ $startcodon = $stopcodon = 0; }
+    if($data =~ /A[01]/){ $startcodon = $stopcodon = 0; $output = 'stdout' if($output =~ /g|show/); }
 
     my $i;
     my $aaseq;
@@ -305,7 +305,7 @@ sub codon_compiler {
 #	msg_send("\t", scalar keys %usage, "\t$total\t$id");
 	msg_send("\n\n");
     }
-    elsif($output =~ /g|show/ && $id !~ /FEATURE/ && $data !~ /A/){
+    elsif($output =~ /g|show/ && $id !~ /FEATURE/ && $data !~ /A[01]/){
 	my $result;
 	foreach (keys %usage){
 	    if($_ =~ /[A-Z\*][a-z]{3}/){ my $amino = substr($_, 0, 1); my $codon = substr($_, 1, 3); $$result{$amino}{$codon} = $usage{$_}; }
@@ -837,11 +837,13 @@ sub icdi {
                 (Ile, Asn, Phe, and Tyr), where the optimal codons seem to be
                 the same in all species, are used.
    -translate   1 when translates using standard codon table (default: 0)
+   -tag         feature tag accessible at $gb->{"FEATURE$i"}->{$tag} (default: 'locus_tag')
 
   Author:
    Haruo Suzuki (haruo@g-language.org)
 
   History:
+   20100311 added -tag option
    20030401 initial posting
 
   References:
@@ -851,12 +853,13 @@ sub icdi {
 =cut
 
 sub fop {
-    &opt_default(output=>'stdout', filename=>'fop.csv', translate=>0);
+    &opt_default(output=>'stdout', filename=>'fop.csv', translate=>0, tag=>'locus_tag');
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $output = opt_val("output");
     my $filename = opt_val("filename");
     my $translate = opt_val("translate");
+    my $tag = opt_val("tag");
 
     # S.cerevisiae
     my %Sc = ('Lcta'=>0,'Lctc'=>0,'Lctg'=>0,'Lctt'=>0,'Ltta'=>0,'Lttg'=>1,
@@ -909,30 +912,22 @@ sub fop {
     foreach (keys %{$binary}){ $degeneracy{substr($_, 0, 1)} ++; }
     
     if($output eq 'stdout'){
-	msg_send("\n\n",
-		 "optimal (1) and nonoptimal (0) codons\n",
-		 "-------------------------------------\n"
-		 );
+	msg_send("\noptimal (1) and nonoptimal (0) codons\n");
 	foreach (sort keys %$binary){ msg_send("$_\t"); }
 	print "\n";
 	foreach (sort keys %$binary){ msg_send("$binary->{$_}\t"); }
 	msg_send("\n\n",
-		 "Frequency of optimal codons (Fop; ratio of optimal codons to synonymous codons)\n",
-		 " Laa: length in amino acids\n",
-		 " Ls: length in silent sites (number of synonymous codons)\n",
-		 "--------------------------------------------\n",
-		 "Laa\tLs\tstart\tstop\tFop\tgene\n",
-		 "--------------------------------------------\n");
+		 "frequency of optimal codons (fop)\n",
+		 "Laa: length in amino acids\n",
+		 "Lc: length in codons used\n",
+		 "Laa\tLc\tfop\t$tag\n");
     }
     if($output eq 'f'){
         mkdir ("data", 0777);
         open(FILE,">>data/$filename");
         #open(FILE,">>$filename");
 	print FILE
-	    "Frequency of optimal codons (Fop; ratio of optimal codons to synonymous codons)\n",
-	    " Laa: length in amino acids\n",
-	    " Ls: length in silent sites (number of synonymous codons)\n",
-	    "Laa,Ls,start codon,stop codon,Fop,gene,product\n";
+	    "Laa,Lc,fop,$tag\n";
     }
 
     foreach my $cds ($gb->cds()){
@@ -945,7 +940,7 @@ sub fop {
 	else { $aaseq = substr($gb->{$cds}->{translation}, 1); }
 
 	my $Laa = length($aaseq);
-	my $Ls;  # length in silent sites
+	my $Lc;  # number of codons used
 	my $fop; # frequency of optimal codons
 	
 	$i = 0;
@@ -954,29 +949,24 @@ sub fop {
 	    substr($aaseq, 0, 1) = '';
 	    my $codon = substr($seq, $i, 3);
 	    if($degeneracy{$aa} > 1 && exists $binary->{$aa.$codon}){
-		$Ls ++;
+		$Lc ++;
 		$fop += $binary->{$aa.$codon};
 	    }
 	    $i += 3;
 	}
-	$gb->{$cds}->{fop} = sprintf "%.4f", $fop / $Ls if($Ls);
-	$gb->{$cds}->{Ls} = $Ls;
+	$gb->{$cds}->{fop} = sprintf "%.4f", $fop / $Lc if($Lc);
+	$gb->{$cds}->{Lc} = $Lc;
 
 	if($output eq 'stdout'){
-	    msg_send("$Laa\t$Ls\t",
-		     $gb->startcodon($cds), "\t",
-		     $gb->stopcodon($cds), "\t",
+	    msg_send("$Laa\t$Lc\t",
 		     $gb->{$cds}->{fop}, "\t",
-		     $gb->{$cds}->{gene}, "\n");
+		     $gb->{$cds}->{$tag}, "\n");
 	}
 	if($output eq 'f'){
 	    print FILE
-		"$Laa,$Ls,",
-		$gb->startcodon($cds), ",",
-		$gb->stopcodon($cds), ",",
+		"$Laa,$Lc,",
 		$gb->{$cds}->{fop}, ",",
-		$gb->{$cds}->{gene}, ",",
-		$gb->{$cds}->{product}, "\n";
+		$gb->{$cds}->{$tag}, "\n";
 	}
     }
     close(FILE);
@@ -999,6 +989,7 @@ sub fop {
                 'stdout' for standard output,
                 'f' for file output in directory "data"
    -filename    output filename (default: 'w_value.csv')
+   -tag         feature tag accessible at $gb->{"FEATURE$i"}->{$tag} (default: 'product')
    -include     regular expression to include genes in a reference set
                 a reference set in several studies are in-built
                 1: Nakamura and Tabata, 2: Sharp and Li, 3: Sakai et al.
@@ -1010,7 +1001,8 @@ sub fop {
    Haruo Suzuki (haruo@g-language.org)
 
   History:
-   20030401 addition of -in & -out option
+   20100311 added -tag option
+   20030401 added -in and -out options
    20010830 first release (Author: Kazuharu "Gaou" Arakawa)
 
   References:
@@ -1023,12 +1015,13 @@ sub fop {
 =cut
 
 sub w_value {
-    &opt_default(output=>'stdout', filename=>"w_value.csv",
+    &opt_default(output=>'stdout', filename=>"w_value.csv", tag=>'product',
 		 include=>'ribosomal.*protein', exclude=>'[Mm]itochondrial');
     my @args=opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $output = opt_val("output");
     my $filename = opt_val("filename");
+    my $tag = opt_val("tag");
     my $include;
     my $exclude = opt_val("exclude");
     
@@ -1042,14 +1035,14 @@ sub w_value {
     if($output eq 'stdout'){
 	msg_send("\n",
 		 "Reference set of highly expressed genes\n",
-		 "start\tend\tgene\tproduct\n");
+		 "start\tend\t$tag\n");
     }
     if($output eq 'f'){
         mkdir ("data", 0777);
         open(FILE,">>data/$filename");
         print FILE
             "Reference set of highly expressed genes\n",
-            "start,end,gene,product\n";
+            "start,end,$tag\n";
     }
     
     my $aaseq;
@@ -1063,8 +1056,7 @@ sub w_value {
     my $cnt;
 
     foreach my $cds ($gb->cds()){
-	next if ($gb->{$cds}->{note} !~ /$include/i &&
-		 $gb->{$cds}->{product} !~ /$include/i); 
+	next if ($gb->{$cds}->{product} !~ /$include/i); 
 	next if ($gb->{$cds}->{product} =~ /$exclude/);
 
 	$cnt ++;
@@ -1082,15 +1074,13 @@ sub w_value {
 	if($output eq 'stdout'){
 	    msg_send($gb->{$cds}->{start}, "\t",
 		     $gb->{$cds}->{end}, "\t",
-		     $gb->{$cds}->{gene}, "\t", 
-		     $gb->{$cds}->{product}, "\n");
+		     $gb->{$cds}->{$tag}, "\n");
 	}
 	if($output eq 'f'){
 	    print FILE 
 		$gb->{$cds}->{start}, ",",
 		$gb->{$cds}->{end}, ",",
-		$gb->{$cds}->{gene}, ",",
-		$gb->{$cds}->{product}, "\n";
+		$gb->{$cds}->{$tag}, "\n";
 	}
     }
 
@@ -1148,6 +1138,7 @@ sub w_value {
                 'f' for file output in directory "data"
    -filename    output filename (default: "cai.csv")
    -translate   1 when translates using standard codon table (default: 0)
+   -tag         feature tag accessible at $gb->{"FEATURE$i"}->{$tag} (default: 'locus_tag')
    -w_output    output option for W value (default: 'stdout')
    -w_filename  filename to output the W values (default: "w_value.csv")
    -w_values    pointer HASH of W values used in CAI calculation
@@ -1159,7 +1150,8 @@ sub w_value {
    Haruo Suzuki (haruo@g-language.org)
 
   History:
-   20030401 addition of -w_absent option
+   20100311 added -tag option
+   20030401 added -w_absent option
    20010830 first release (Author: Kazuharu "Gaou" Arakawa)
 
   References:
@@ -1170,13 +1162,14 @@ sub w_value {
 =cut
 
 sub cai {
-    &opt_default(output=>'stdout', filename=>"cai.csv", translate=>0,
+    &opt_default(output=>'stdout', filename=>"cai.csv", translate=>0, tag=>'locus_tag',
 		 w_output=>'stdout', w_filename=>"w_value.csv", w_absent=>"-");
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $output = opt_val("output");
     my $filename = opt_val("filename");
     my $translate = opt_val("translate");
+    my $tag = opt_val("tag");
     my $w_output = opt_val("w_output");
     my $w_fname = opt_val("w_filename");
     my $w_abs = opt_val("w_absent");
@@ -1192,12 +1185,12 @@ sub cai {
     
     if($output eq 'stdout'){
 	msg_send("\nCodon Adaptation Index (CAI) - geometric mean of W values\n",
-		 "start\tend\tcai\tgene\n");
+		 "cai\t$tag\n");
     }
     if($output eq 'f'){
         mkdir ("data", 0777);
         open(FILE,">>data/$filename");
-        print FILE "start,end,cai,gene,product\n";
+        print FILE "cai,$tag\n";
     }
 
     foreach my $cds ($gb->cds()){
@@ -1210,30 +1203,25 @@ sub cai {
 	else { $aaseq = substr($gb->{$cds}->{translation}, 1); }
 	
 	my $cai;
-	my $L;
+	my $Lc;
 	for($i = 0; $i * 3 <= length($seq) - 3; $i ++){
 	    my $aa = substr($aaseq, $i, 1);
 	    $codon = substr($seq, $i * 3, 3);
 	    if($w_val->{$aa.$codon} > 0){ $cai += log($w_val->{$aa.$codon}); }
 	    elsif($w_abs){ next if($w_abs eq '-'); $cai += log($w_abs); }
 	    else { $cai = 999; last; }
-	    $L ++;
+	    $Lc ++;
 	}
-	$gb->{$cds}->{cai} = ($cai == 999) ? 0 : sprintf "%.4f", exp($cai/$L) if($L);
+	$gb->{$cds}->{cai} = ($cai == 999) ? 0 : sprintf "%.4f", exp($cai/$Lc) if($Lc);
 
 	if($output eq 'stdout'){
-	    msg_send($gb->{$cds}->{start}, "\t",
-		     $gb->{$cds}->{end}, "\t",
-		     $gb->{$cds}->{cai}, "\t",
-		     $gb->{$cds}->{gene}, "\n");
+	    msg_send($gb->{$cds}->{cai}, "\t",
+		     $gb->{$cds}->{$tag}, "\n");
 	}
 	if($output eq 'f'){
 	    print FILE
-		$gb->{$cds}->{start}, ",",
-		$gb->{$cds}->{end}, ",",
 		$gb->{$cds}->{cai}, ",",
-		$gb->{$cds}->{gene}, ",",
-		$gb->{$cds}->{product}, "\n";
+		$gb->{$cds}->{$tag}, "\n";
 	}
     }
     close(FILE);
@@ -1246,7 +1234,7 @@ sub cai {
 
   Description:
    This program calculates codon usage differences between gene classes for identifying 
-   predicted highly expressed (PHX) genes. The "expression measure" of a gene, E(g), 
+   predicted highly expressed (PHX) genes. The "expression measure" of a gene, E_g, 
    will be accessible at $gb->{"FEATURE$i"}->{E_g}.
 
   Usage:
@@ -1263,18 +1251,20 @@ sub cai {
    -translate   1 when translates using standard codon table (default: 0)
    -del_key     regular expression to delete key (i.e. amino acids and nucleotides)
                 (default: '[^ACDEFGHIKLMNPQRSTVWYacgtU]')
+   -tag         feature tag accessible at $gb->{"FEATURE$i"}->{$tag} (default: 'locus_tag')
 
   Author:
    Haruo Suzuki (haruo@g-language.org)
 
   History:
+   20100311 added -tag option
    20030401 initial posting
 
   Informations stored in the G instance:
-   $gb->{"FEATURE$i"}->{BgC}: B(g|C)
-   $gb->{"FEATURE$i"}->{BgH}: B(g|H)
-   $gb->{"FEATURE$i"}->{E_g}: E(g) [B(g|C) / B(g|H)]
-   $gb->{"FEATURE$i"}->{phx}: 1 if a gene is PHX [i.e. E(g) > 1.05]
+   $gb->{"FEATURE$i"}->{BgC}: codon bias relative to a collection of all genes
+   $gb->{"FEATURE$i"}->{BgH}: codon bias relative to a set of highly expressed genes
+   $gb->{"FEATURE$i"}->{E_g}: expression measure E_g = BgC / BgH
+   $gb->{"FEATURE$i"}->{phx}: 1 if a gene is PHX (E_g > 1.05)
 
   References:
    Karlin and Mrazek (2000) J.Bacteriol. 182(18):5238-5250.
@@ -1285,7 +1275,7 @@ sub cai {
 =cut
 
 sub phx {
-    &opt_default(output=>'stdout', filename=>"phx.csv", usage=>"",
+    &opt_default(output=>'stdout', filename=>"phx.csv", usage=>"", tag=>'locus_tag',
 		 translate=>0, del_key=>'[^ACDEFGHIKLMNPQRSTVWYacgtU]');
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
@@ -1294,6 +1284,7 @@ sub phx {
     my $translate = opt_val("translate");
     my $del_key = opt_val("del_key");
     my $High = opt_val("usage");
+    my $tag = opt_val("tag");
     
     my $cds;
     unless($High){
@@ -1316,17 +1307,17 @@ sub phx {
     }
     
     if($output eq 'stdout'){
-        msg_send("\n\nPredicted highly expressed (PHX) genes\n",
-		 " B(g|C): codon usage difference relative to collection of all genes\n",
-		 " B(g|H): codon usage difference relative to a set of highly expressed genes\n",
-		 " E(g): predicted expression level [B(g|C) / B(g|H)]\n",
-		 " PHX = 1 if E(g) > 1.05\n",
-		 "start\tend\tB(g|C)\tB(g|H)\tE(g)\tPHX\tgene\n");
+        msg_send("\nPredicted highly expressed (PHX) genes\n",
+		 " BgC: codon usage difference from a collection of all genes\n",
+		 " BgH: codon usage difference from a collection of highly expressed genes\n",
+		 " E_g: expression measure (BgC/BgH)\n",
+		 " phx = 1 if E_g > 1.05\n",
+		 "BgC\tBgH\tE_g\tphx\t$tag\n");
     }
     if($output eq 'f'){
         mkdir ("data", 0777);
         open(FILE,">>data/$filename");
-        print FILE "start,end,B(g|C),B(g|H),E(g),PHX,gene,product\n";
+        print FILE "BgC,BgH,E_g,phx,$tag\n";
     }
     
     foreach $cds ($gb->cds()){
@@ -1356,24 +1347,19 @@ sub phx {
 	}
 
 	if($output eq 'stdout'){
-	    msg_send($gb->{$cds}->{start}, "\t",
-		     $gb->{$cds}->{end}, "\t",
-		     $gb->{$cds}->{BgC}, "\t",
+	    msg_send($gb->{$cds}->{BgC}, "\t",
 		     $gb->{$cds}->{BgH}, "\t",
 		     $gb->{$cds}->{E_g}, "\t",
 		     $gb->{$cds}->{phx}, "\t",
-		     $gb->{$cds}->{gene}, "\n");
+		     $gb->{$cds}->{$tag}, "\n");
 	}
 	if($output eq 'f'){
 	    print FILE
-		$gb->{$cds}->{start}, ",",
-		$gb->{$cds}->{end}, ",",
 		$gb->{$cds}->{BgC}, ",",
 		$gb->{$cds}->{BgH}, ",",
 		$gb->{$cds}->{E_g}, ",",
 		$gb->{$cds}->{phx}, ",",
-		$gb->{$cds}->{gene}, ",",
-		$gb->{$cds}->{product}, "\n";
+		$gb->{$cds}->{$tag}, "\n";
 	}
     }
     close(FILE);
@@ -1408,11 +1394,13 @@ sub phx {
                 'f' for file output in directory "data"
    -filename    output filename (default: 'aaui.csv')
    -id          ID of a group of genes or a single gene (default: 'All')
+   -tag         feature tag accessible at $gb->{"FEATURE$i"}->{$tag} (default: 'locus_tag')
 
   Author:
    Haruo Suzuki (haruo@g-language.org)
 
   History:
+   20100311 added -tag option
    20030401 initial posting
 
   References:
@@ -1422,12 +1410,13 @@ sub phx {
 =cut
 
 sub aaui {
-    &opt_default(output=>'stdout', filename=>'aaui.csv', id=>'All');
+    &opt_default(output=>'stdout', filename=>'aaui.csv', id=>'All', tag=>'locus_tag');
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $output = opt_val("output");
     my $filename = opt_val("filename");
     my $id = opt_val("id");
+    my $tag = opt_val("tag");
     
     my $aaseq;
     if($id =~ /FEATURE/){ # for a single gene
@@ -1490,16 +1479,15 @@ sub aaui {
     my $gene = ($id =~ /FEATURE/) ? $gb->{$id}->{gene} : "{$id}";
     if($output eq 'stdout'){
 	unless($gb->{header_aaui}){
-	    msg_send("\n\n",
-		     "Indices of amino acid usage (excluding formylmethionine)\n",
+	    msg_send("\n",
+		     "amino acid usage indices (excluding formylmethionine)\n",
 		     " Laa: length in amino acids\n",
 		     " ndaa: number of different amino acids\n",
 		     " Haau: entropy of amino acid usage\n",
 		     " mmw: mean molecular weight\n",
 		     " gravy: mean hydropathic indices of each amino acid\n",
-		     "----------------------------------------------------------------------------\n",
-		     "Laa\tndaa\tHaau\tmmw\tgravy\taroma\tacidic\tbasic\tneutral\tgene\n",
-		     "----------------------------------------------------------------------------\n");
+		     " aroma: relative frequency of aromatic amino acids\n",
+		     "Laa\tndaa\tHaau\tmmw\tgravy\taroma\t$tag\n");
 	    $gb->{header_aaui} = 1;
 	}
 	msg_send(
@@ -1509,10 +1497,7 @@ sub aaui {
 		 $gb->{$id}->{mmw}, "\t",
 		 $gb->{$id}->{gravy}, "\t",
 		 $gb->{$id}->{aroma}, "\t",
-		 $gb->{$id}->{acidic}, "\t",
-		 $gb->{$id}->{basic}, "\t",
-		 $gb->{$id}->{neutral}, "\t",
-		 $gene, "\n");
+		 $gb->{$id}->{$tag}, "\n");
     }
     if($output eq 'f'){
         mkdir ("data", 0777);
@@ -1520,13 +1505,7 @@ sub aaui {
         #open(FILE,">>$filename");
 	unless($gb->{header_aaui}){
 	    print FILE
-		"Indices of amino acid usage (excluding formylmethionine)\n",
-		" Laa: length in amino acids\n",
-		" ndaa: number of different amino acids\n",
-		" Haau: entropy in amino acid usage\n",
-		" mmw: mean molecular weight\n",
-		" gravy: mean hydropathic indices of each amino acid\n",
-		"Laa,ndaa,Haau,mmw,gravy,aroma,acidic,basic,neutral,polar,gene\n";
+		"Laa,ndaa,Haau,mmw,gravy,aroma,$tag\n";
 	    $gb->{header_aaui} = 1;
 	}
 	print FILE
@@ -1536,11 +1515,7 @@ sub aaui {
 	    $gb->{$id}->{mmw}, ",",
 	    $gb->{$id}->{gravy}, ",",
 	    $gb->{$id}->{aroma}, ",",
-	    $gb->{$id}->{acidic}, ",",
-	    $gb->{$id}->{basic}, ",",
-	    $gb->{$id}->{neutral}, ",",
-	    $gb->{$id}->{polar}, ",",
-	    $gene, "\n";
+	    $gb->{$id}->{$tag}, "\n";
     }
     close(FILE);
 }
@@ -1555,16 +1530,12 @@ sub aaui {
    (excluding start and stop codons), and inputs in the G instance.
    Informations stored in the G instance are:
     $gb->{$id}->{"acgt$p"}: total number of bases (A+T+G+C)
-    $gb->{$id}->{"a_c$p"}: A content [A/(A+T+G+C)]
-    $gb->{$id}->{"c_c$p"}: C content [C/(A+T+G+C)]
-    $gb->{$id}->{"g_c$p"}: G content [G/(A+T+G+C)]
-    $gb->{$id}->{"t_c$p"}: T content [T/(A+T+G+C)]
-    $gb->{$id}->{"ryr$p"}: purine/pyrimidine ratio [(A+G)/(T+C)]
-    $gb->{$id}->{"gac$p"}: G+A content [(G+A)/(A+T+G+C)]
-    $gb->{$id}->{"gcc$p"}: G+C content [(G+C)/(A+T+G+C)]
-    $gb->{$id}->{"gtc$p"}: G+T content [(G+T)/(A+T+G+C)]
-    $gb->{$id}->{"gcs$p"}: GC skew [(G-C)/(G+C)]
-    $gb->{$id}->{"tas$p"}: TA skew [(T-A)/(T+A)]
+    $gb->{$id}->{"ryr$p"}: purine/pyrimidine ratio (A+G)/(T+C)
+    $gb->{$id}->{"gac$p"}: G+A content (G+A)/(A+T+G+C)
+    $gb->{$id}->{"gcc$p"}: G+C content (G+C)/(A+T+G+C)
+    $gb->{$id}->{"gtc$p"}: G+T content (G+T)/(A+T+G+C)
+    $gb->{$id}->{"gcs$p"}: GC skew (G-C)/(G+C)
+    $gb->{$id}->{"tas$p"}: TA skew (T-A)/(T+A)
     $gb->{$id}->{"Hbu$p"}: entropy of base usage
     $gb->{header_bui}
 
@@ -1585,18 +1556,20 @@ sub aaui {
                 '1' when assessing base usage at 1st position of codons
                 '2' when assessing base usage at 2nd position of codons
                 '3' when assessing base usage at 3rd position of codons
+   -tag         feature tag accessible at $gb->{"FEATURE$i"}->{$tag} (default: 'locus_tag')
 
   Author:
    Haruo Suzuki (haruo@g-language.org)
 
   History:
+   20100311 added -tag option
    20030401 initial posting
 
 =cut
 
 sub bui {
     &opt_default(output=>'stdout',filename=>'bui.csv',id=>'All',translate=>0,
-		 del_key=>'[^ACDEFGHIKLMNPQRSTVWYacgtU]',position=>'');
+		 del_key=>'[^ACDEFGHIKLMNPQRSTVWYacgtU]',position=>'',tag=>'locus_tag');
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $output = opt_val("output");
@@ -1605,6 +1578,7 @@ sub bui {
     my $translate = opt_val("translate");
     my $del_key = opt_val("del_key");
     my $p = opt_val("position");
+    my $tag = opt_val("tag");
 
     my $seq;
     my $aaseq;
@@ -1656,10 +1630,6 @@ sub bui {
 	$g = $seq =~ tr/g/g/;
 	$t = $seq =~ tr/t/t/;
 	$gb->{$id}->{"acgt$p"} = $a+$c+$g+$t;
-	$gb->{$id}->{"a_c$p"} = sprintf "%.4f", $a / ($a+$c+$g+$t);
-	$gb->{$id}->{"c_c$p"} = sprintf "%.4f", $c / ($a+$c+$g+$t);
-	$gb->{$id}->{"g_c$p"} = sprintf "%.4f", $g / ($a+$c+$g+$t);
-	$gb->{$id}->{"t_c$p"} = sprintf "%.4f", $t / ($a+$c+$g+$t);
 	
 	if($t+$c){ $gb->{$id}->{"ryr$p"} = sprintf "%.4f", ($a+$g) / ($t+$c); }
 	else { &msg_error("t + c = 0 in $id\n"); }
@@ -1695,30 +1665,24 @@ sub bui {
     my $gene = ($id =~ /FEATURE/) ? $gb->{$id}->{gene} : "{$id}";
     if($output eq 'stdout'){
 	unless($gb->{header_bui}){
-	    msg_send("\n\n",
-		     "Base usage indices (excluding start and stop codons)\n",
-		     " ACGT: A$p + T$p + G$p + C$p\n",
-		     " GAc: (G$p + A$p)/(A$p + T$p + G$p + C$p)\n",
-		     " GCc: (G$p + C$p)/(A$p + T$p + G$p + C$p)\n",
-		     " GTc: (G$p + T$p)/(A$p + T$p + G$p + C$p)\n",
-		     " GCs: (G$p - C$p)/(G$p + C$p)\n",
-		     " TAs: (T$p - A$p)/(T$p + A$p)\n",
+	    msg_send("\n",
+		     "base usage indices (excluding start and stop codons)\n",
+		     " acgt: A$p + T$p + G$p + C$p\n",
+		     " ryr: purine/pyrimidine ratio (A$p + G$p)/(T$p + C$p)\n",
+		     " gcc: G+C content (G$p + C$p)/(A$p + T$p + G$p + C$p)\n",
+		     " gcs: GC skew (G$p - C$p)/(G$p + C$p)\n",
+		     " tas: TA skew (T$p - A$p)/(T$p + A$p)\n",
 		     " Hbu: entropy of base usage\n",
-		     "-------------------------------------------------------------\n",
-		     "start\tend\tACGT\tGAc\tGCc\tGTc\tGCs\tTAs\tHbu\tgene\n",
-		     "-------------------------------------------------------------\n");
+		     "acgt\tryr\tgcc\tgcs\ttas\tHbu\t$tag\n");
 	    $gb->{header_bui} = 1;
 	}
-	msg_send($gb->{$id}->{start}, "\t",
-		 $gb->{$id}->{end}, "\t",
-		 $gb->{$id}->{"acgt$p"}, "\t",
-		 $gb->{$id}->{"gac$p"}, "\t",
+	msg_send($gb->{$id}->{"acgt$p"}, "\t",
+		 $gb->{$id}->{"ryr$p"}, "\t",
 		 $gb->{$id}->{"gcc$p"}, "\t",
-		 $gb->{$id}->{"gtc$p"}, "\t",
 		 $gb->{$id}->{"gcs$p"}, "\t",
 		 $gb->{$id}->{"tas$p"}, "\t",
 		 $gb->{$id}->{"Hbu$p"}, "\t",
-		 $gene, "\n");
+		 $gb->{$id}->{"$tag"}, "\n");
     }
     if($output eq 'f'){
         mkdir ("data", 0777);
@@ -1726,20 +1690,17 @@ sub bui {
         #open(FILE,">>$filename");
 	unless($gb->{header_bui}){
 	    print FILE
-		"start,end,ACGT,GAc,GCc,GTc,GCs,TAs,Hbu,gene\n";
+		"acgt,ryr,gcc,gcs,tas,Hbu,$tag\n";
 	    $gb->{header_bui} = 1;
 	}
 	print FILE
-	    $gb->{$id}->{start}, ",",
-	    $gb->{$id}->{end}, ",",
 	    $gb->{$id}->{"acgt$p"}, ",",
-	    $gb->{$id}->{"gac$p"}, ",",
+	    $gb->{$id}->{"ryr$p"}, ",",
 	    $gb->{$id}->{"gcc$p"}, ",",
-	    $gb->{$id}->{"gtc$p"}, ",",
 	    $gb->{$id}->{"gcs$p"}, ",",
 	    $gb->{$id}->{"tas$p"}, ",",
 	    $gb->{$id}->{"Hbu$p"}, ",",
-	    $gene, "\n";
+	    $gb->{$id}->{$tag}, "\n";
 	close(FILE);
     }
 }
@@ -1917,39 +1878,43 @@ sub dinuc {
    -translate   1 when translates using standard codon table (default: 0)
    -del_key     regular expression to delete key (i.e. amino acids and nucleotides)
                 (default: '[^ACDEFGHIKLMNPQRSTVWYacgtU]')
+   -tag         feature tag accessible at $gb->{"FEATURE$i"}->{$tag} (default: 'locus_tag')
 
   Author: 
    Haruo Suzuki (haruo@g-language.org)
 
   History:
+   20100311 added -tag option
    20070701 initial posting
 
   References:
    Suzuki H et al. (2004) Gene. 23;335:19-23.
-   Suzuki H et al. (2007) http://www.hindawi.com/GetArticle.aspx?doi=10.1155/2007/61374
+   Suzuki H et al. (2007) EURASIP J Bioinform Syst Biol. 2007:61374.
 
   Requirements: sub codon_compiler;
 
 =cut
 
 sub Ew {
-    &opt_default(output=>'stdout', filename=>'Ew.csv', translate=>0, del_key=>'[^ACDEFGHIKLMNPQRSTVWYacgtU]');
+    &opt_default(output=>'stdout', filename=>'Ew.csv', translate=>0, del_key=>'[^ACDEFGHIKLMNPQRSTVWYacgtU]', tag=>'locus_tag');
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $output = opt_val("output");
     my $filename = opt_val("filename");
     my $translate = opt_val("translate");
     my $del_key = opt_val("del_key");
-    
+    my $tag = opt_val("tag");
+
     if($output eq 'stdout'){
-        msg_send("\n\nThe 'weighted sum of relative entropy' (Ew)\n",
-                 "Ew\tgene\n");
+        msg_send("\nThe 'weighted sum of relative entropy' (Ew)\n",
+		 "Ew ranges from 0 (maximum bias) to 1 (no bias).\n",
+                 "Ew\t$tag\n");
     }
     if($output eq 'f'){
 	mkdir ("data", 0777);
         open(FILE,">>data/$filename");
         print FILE
-            "Ew,gene\n";
+            "Ew,$tag\n";
     }
     
     foreach my $cds ($gb->cds()){
@@ -1969,12 +1934,12 @@ sub Ew {
 	
         if($output eq 'stdout'){
             msg_send($gb->{$cds}->{Ew}, "\t",
-                     $gb->{$cds}->{gene}, "\n");
+                     $gb->{$cds}->{$tag}, "\n");
         }
         if($output eq 'f'){
             print FILE
                 $gb->{$cds}->{Ew}, ",",
-                $gb->{$cds}->{gene}, "\n";
+                $gb->{$cds}->{$tag}, "\n";
         }
     }
     close(FILE);
@@ -2000,11 +1965,13 @@ sub Ew {
                 'stdout' for standard output,
                 'f' for file output in directory "data"
    -filename    output filename (default: 'P2.csv')
+   -tag         feature tag accessible at $gb->{"FEATURE$i"}->{$tag} (default: 'locus_tag')
 
-  Author: 
+  Author:
    Haruo Suzuki (haruo@g-language.org)
 
   History:
+   20100311 added -tag option
    20070701 initial posting
 
   References:
@@ -2015,21 +1982,22 @@ sub Ew {
 =cut
 
 sub P2 {
-    &opt_default(output=>'stdout', filename=>'P2.csv');
+    &opt_default(output=>'stdout', filename=>'P2.csv', tag=>'locus_tag');
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $output = opt_val("output");
     my $filename = opt_val("filename");
+    my $tag = opt_val("tag");
     
     if($output eq 'stdout'){
-        msg_send("\n\nThe P2 index\n",
-                 "P2\tgene\n");
+        msg_send("\nThe P2 index\n",
+                 "P2\t$tag\n");
     }
     if($output eq 'f'){
 	mkdir ("data", 0777);
         open(FILE,">>data/$filename");
         print FILE
-            "P2,gene\n";
+            "P2,$tag\n";
     }
     
     my ($WWC, $SST, $WWY, $SSY);
@@ -2047,12 +2015,12 @@ sub P2 {
 	
         if($output eq 'stdout'){
             msg_send($gb->{$cds}->{P2}, "\t",
-                     $gb->{$cds}->{gene}, "\n");
+                     $gb->{$cds}->{$tag}, "\n");
         }
         if($output eq 'f'){
             print FILE
                 $gb->{$cds}->{P2}, ",",
-                $gb->{$cds}->{gene}, "\n";
+                $gb->{$cds}->{$tag}, "\n";
         }
     }
     close(FILE);
@@ -2265,7 +2233,7 @@ sub codon_mva {
 			     gfd[zz > zc & gfd == "nd"] = "Expression"
 			     res = gfd
 			     res[substr(res,nchar(res),nchar(res)) == ")"] = "two"
-			     write.csv(rbind(gpd,gfd,res,out,key,val,zz), file = "haruo$data.csv", quote = F)
+			     write.csv(rbind(gpd,gfd,res,out,key,val,zz), file = "table$data.csv", quote = F)
 			     # returned value
 			     as.vector(c(mean(rda),mean(eda),mean(rdh),mean(edh), contribution, z.score, rr))
 			     /
@@ -2339,7 +2307,7 @@ sub codon_mva {
 =cut
 
 sub codon_counter {
-    warn('This method is deprecated. Use &codon_compiler(@_, -data=>'C0') instead.');
+    warn('This method is deprecated. Use &codon_compiler(@_, -data=>"C0") instead.');
     return &codon_compiler(@_, -data=>'C0'); 
 }
 
@@ -2371,12 +2339,11 @@ sub codon_counter {
 =cut
 
 sub amino_counter {
-    warn('This method is deprecated. Use &codon_compiler(@_, -data=>'A0') instead.');
-    return &codon_compiler(@_, -data=>'A0'); 
+    warn('This method is deprecated. Use &codon_compiler(@_, -data=>"A0") instead.');
+    return &codon_compiler(@_, -data=>'A0');
 }
 
 
-
 =head2 codon_usage
 
   Name: codon_usage   -   calculates codon usage
@@ -2407,7 +2374,7 @@ sub amino_counter {
 =cut
 
 #sub codon_usage {
-#    warn('This method is deprecated. Use &codon_compiler(@_, -data=>'C2') instead.');
+#    warn('This method is deprecated. Use &codon_compiler(@_, -data=>"C2") instead.');
 #    return &codon_compiler(@_, -data=>'C2'); 
 #}
 
