@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Consensus.pm,v 1.2 2002/07/30 17:40:56 gaou Exp $
@@ -67,12 +67,40 @@ G::Seq::Consensus - Analysis methods related to sequence consensus
 #     Let the code begin...
 #::::::::::::::::::::::::::::::
 
-#base_counter ver.20010623-01
-#scripting by Koya Mori(mory@g-language.org)
-#This program analyzes bases around start codon.
-#(hash result)=&bun(pointer Genome,  string position,  int pattern_length,  int upstream_limit,  int downstream_limit,  boolean debug);
+=head2 base_counter
+
+  Name: base_counter   -   creates a position weight matrix of oligomers around start codon
+
+  Description:
+    This function creates a position weight matrix (PWM) of 
+    oligomers of specified length around the start codon of all 
+    genes in the given genome.
+
+  Usage:
+    $structure = base_counter($genome);
+
+    Here the $structure is a reference to the array of hashes, intended for 
+    internal usage.
+
+  Options:
+     -position   either "start" (around start codon) or "end" (around stop codon)
+                 to create the PWM (default: "start");
+     -PatLen     length of oligomer to count (default: 3)
+     -upstream   length upstream of specified position to create PWM (default: 30)
+     -downstream length downstream of specified position to create PWM (default: 30)
+     -filename   outfile file name (default: consensus.csv)
+     -output     "stdout" for standard output, "f" for file
+
+  Author: 
+    Koya Mori(mory@g-language.org)
+
+  History:
+   20010623-01 initial posting (ported from ATG7 program, "bun")
+
+=cut
+
 sub base_counter{
-    &opt_default(output=>"stdout",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"end");
+    &opt_default(output=>"stdout",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
@@ -249,30 +277,41 @@ sub _base_printer{
 }
 
 
-#logmod ver.20010714-01
-#scripting by Koya Mori(mory@g-language.org)
-#This program calculate log2.
-#(double result)=&logmod(double number,  boolean debug);
 sub logmod{
-    my $x=shift;
-
-    if($x == 0){
-	return 0;
-    } 
-    else{
-	return log($x);
-    }
+    return $_[0] == 0 ? 0 : log($_[0]);
 }
 
 
-#buns::ent in Atg7
-#base_entropy ver.20010623-01
-#scripting by Koya Mori(mory@g-language.org)
-#This program calculate entropy of base_counter.
-#need &base_counter.
-#(array result)=&base_entropy(hash base_counter_result,  boolean debug);
+=head2 base_entropy
+
+  Name: base_entropy   -   calculates and graphs the sequence conservation using Shanon uncertainty (entropy)
+
+  Description:
+    This function calculates and graphs the sequence conservation in regions 
+    around the start/stop codons using Shanon uncertainty (entropy).
+
+  Usage:
+    ($position, "entropy", $upstream-length, $downstream-length, @entropyvalues) = base_entropy($genome);
+
+  Options:
+     -position   either "start" (around start codon) or "end" (around stop codon)
+                 to create the PWM (default: "start");
+     -PatLen     length of oligomer to count (default: 3)
+     -upstream   length upstream of specified position to create PWM (default: 30)
+     -downstream length downstream of specified position to create PWM (default: 30)
+     -filename   outfile file name (default: consensus.csv)
+     -output     "g" for graph, "f" for file, "show" for display
+
+  Author: 
+    Koya Mori(mory@g-language.org)
+
+  History:
+    20010623-01 initial posting (ported from ATG7 program, "buns::ent")
+
+=cut
+
 sub base_entropy{
-    &opt_default(output=>"show",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"end");
+    &opt_default(output=>"show",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
@@ -346,18 +385,40 @@ sub base_entropy{
 
     $tmp="entropy";
     @entropy=($position,$tmp,$upstream,$downstream-1,@entropy);
-    return \@entropy;
+    return @entropy;
 }
 
 
-#buns::rseq in Atg7
-#base_informarion_content ver.20010707-01
-#scripting by Koya Mori(mory@g-language.org)
-#This program calculate information content of base_counter.
-#need &base_counter.
-#(array result)=&base_information_content(hash base_counter_result,  boolean debug);
+=head2 base_information_content
+
+  Name: base_information_content   -   calculates and graphs the sequence conservation using information content
+
+  Description:
+    This function calculates and graphs the sequence conservation in regions 
+    around the start/stop codons using information content.
+
+  Usage:
+    ($position, "IC", $upstream-length, $downstream-length, @entropyvalues) = base_information_content($genome);
+
+  Options:
+     -position   either "start" (around start codon) or "end" (around stop codon)
+                 to create the PWM (default: "start");
+     -PatLen     length of oligomer to count (default: 3)
+     -upstream   length upstream of specified position to create PWM (default: 30)
+     -downstream length downstream of specified position to create PWM (default: 30)
+     -filename   outfile file name (default: consensus.csv)
+     -output     "g" for graph, "f" for file, "show" for display
+
+  Author: 
+    Koya Mori(mory@g-language.org)
+
+  History:
+    20010707-01 initial posting (ported from ATG7 program, "buns::rseq")
+
+=cut
+
 sub base_information_content{
-    &opt_default(output=>"show",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"end");
+    &opt_default(output=>"show",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
@@ -432,12 +493,34 @@ sub base_information_content{
 }
 
 
-#buns::ic in Atg7
-#base_relative_entropy ver.20010714-01
-#scripting by Koya Mori(mory@g-language.org)
-#This program calculate relative entropy of base_counter.
-#need &base_counter.
-#(array result)=&base_information_content(hash base_counter_result,  boolean debug);
+=head2 base_relative_entropy
+
+  Name: base_relative_entropy   -   calculates and graphs the sequence conservation using Kullback-Leibler divergence (relative entropy)
+
+  Description:
+    This function calculates and graphs the sequence conservation in regions 
+    around the start/stop codons using Kullback-Leibler divergence (relative entropy).
+
+  Usage:
+    ($position, "RE", $upstream-length, $downstream-length, @entropyvalues) = base_relative_entropy($genome);
+
+  Options:
+     -position   either "start" (around start codon) or "end" (around stop codon)
+                 to create the PWM (default: "start");
+     -PatLen     length of oligomer to count (default: 3)
+     -upstream   length upstream of specified position to create PWM (default: 30)
+     -downstream length downstream of specified position to create PWM (default: 30)
+     -filename   outfile file name (default: consensus.csv)
+     -output     "g" for graph, "f" for file, "show" for display
+
+  Author: 
+    Koya Mori(mory@g-language.org)
+
+  History:
+    20010714-01 initial posting (ported from ATG7 program, "buns::ic")
+
+=cut
+
 sub base_relative_entropy{
     &opt_default(output=>"show",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"end");
     my @args=opt_get(@_);
@@ -503,13 +586,39 @@ sub base_relative_entropy{
 }
 
 
-#base_z_value ver.20010714-01
-#scripting by Koya Mori(mory@g-language.org)
-#This program calculates Z value of base_counter.
-#need &base_counter.
-#(array result)=&base_z_value(hash base_counter_result,  int limit,  boolean debug);
+=head2 base_z_value
+
+  Name: base_z_value   -   extracts conserved oligomers per position using Z-score
+
+  Description:
+    This function calculates and extracts conserved oligomers per position using Z-score,
+    in regions around the start/stop codons using Z-score
+
+  Usage:
+    $structure = base_z_value($genome);
+
+    Here the $structure is a reference to the array of hashes, intended for 
+    internal usage.
+
+  Options:
+     -limit      rank threshold for showing the conserved oligomer (default: 5)
+     -position   either "start" (around start codon) or "end" (around stop codon)
+                 to create the PWM (default: "start");
+     -PatLen     length of oligomer to count (default: 3)
+     -upstream   length upstream of specified position to create PWM (default: 30)
+     -downstream length downstream of specified position to create PWM (default: 30)
+     -output     "stdout" for standard output, "f" for file
+
+  Author: 
+    Koya Mori(mory@g-language.org)
+
+  History:
+    20010714-01 initial posting (ported from ATG7 program, "buns::ic")
+
+=cut
+
 sub base_z_value{
-    &opt_default(limit=>5,output=>"stdout",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"end");
+    &opt_default(limit=>5,output=>"stdout",filename=>"consensus.csv",PatLength=>3,upstream=>30,downstream=>30,position=>"start");
     my @args=opt_get(@_);
 
     my $gb=opt_as_gb(shift @args);
