@@ -158,11 +158,16 @@ my $pi = 4 * atan2(1,1);
  Options:
     -gmap           1 to create Google Map view, 0 to only generate PNG image
                     (default:1)
+    -optionalText   optional text to describe the genome.
 
   Author: 
     Nobuhiro Kido (t07505nk@sfc.keio.ac.jp)
 
   History:
+   20100318-01 major update including support for linear chromosomes,
+               change of gene color by strand in circular genomes,
+               addition of accession number and optional text,
+               updated color coding for GC content and GC skew.
    20080414-01 added gmap option
    20071023-01 initial posting
 
@@ -172,16 +177,17 @@ my $pi = 4 * atan2(1,1);
 sub circular_map{
     opt_default("gmap"=>1);
     my @args = opt_get(@_);
-    my $gb = opt_as_gb(shift @args);
+    my $gb   = opt_as_gb(shift @args);
     my $gmap = opt_val("gmap");
-    mkdir( "data", 0777 );
-    mkdir( "graph",  0777 );
+    my $optionalText = opt_val("optionalText");
+    mkdir( "data",  0777 );
+    mkdir( "graph", 0777 );
 
     msg_error("Map generation will take a few minutes. Please make yourself comfortable until the image is generated ;-\)\n\n") if ($gmap);
 
     my $wfile = $gb->{LOCUS}->{id};
     my ( $ori, $ter ) = find_ori_ter( $gb, -output => "/dev/null" );
-    my $bp_length = length( $gb->{SEQ} );
+    my $bp_length = int(length( $gb->{SEQ} ));
     my ( $count_a, $count_b );
 
     my $svg = SVG->new(
@@ -204,6 +210,40 @@ sub circular_map{
           { 'stroke' => '#ff9966', 'fill' => 'none', 'stroke-width' => 200 }
     );
 
+    $svg->circle(
+		 id  => "cdsringbackground",
+		 cx  => $cx,
+		 cy  => $cy,
+		 r   => 3250,
+		 'stroke' => '#222222', 
+		 'opacity' => 0.8,
+		 'fill' => 'none', 
+		 'stroke-width' => 500
+		 );
+
+    my $ry  = $cy - 3250; #middle = 350
+
+    if($gb->{LOCUS}->{circular} == 0){
+	my $r = 3550;
+	my ( $rx, $ry2 ) = &get_pos( int(length($gb->{SEQ}) * 47/48), $r, $bp_length );
+	$svg->polygon(
+		      points=>"$cx,$ry $rx,$ry2 $cx,$ry2",
+		      stroke=>"#000000",
+		      fill=>'#000000',
+		      'stroke-width'=>30
+		      );
+	
+	$r = 2950;
+	( $rx, $ry2 ) = &get_pos( int(length($gb->{SEQ}) * 47/48), $r, $bp_length );
+	$svg->polygon(
+		      points=>"$cx,$ry $rx,$ry2 $cx,$ry2",
+		      stroke=>"#000000",
+		      fill=>'#000000',
+		      'stroke-width'=>30
+		      );
+    }
+
+    
     #$y->circle(cx=>$cx, cy=>$cy,r=>6500, id=>'main_c');
 
     my $scla = $svg->group(
@@ -277,11 +317,22 @@ sub circular_map{
         style =>
           { 'stroke' => '#ff9933', 'fill' => 'none', 'stroke-width' => 3 }
     );
+    my $gcskc = $svg->group(
+        id => 'gccontent_b',
+        style =>
+          { 'stroke' => '#cc6611', 'fill' => 'none', 'stroke-width' => 3 }
+    );
+
     my $gcskb = $svg->group(
         id => 'gcskew',
         style =>
           { 'stroke' => '#ccff66', 'fill' => 'none', 'stroke-width' => 3 }
     );
+    my $gcskd = $svg->group(
+        id => 'gcskew_b',
+        style =>
+          { 'stroke' => '#aadd44', 'fill' => 'none', 'stroke-width' => 3 }
+    );
 
     my $oriter = $svg->group(
         id => 'ori_ter',
@@ -302,40 +353,73 @@ sub circular_map{
         'font-family' => 'Gill Sans'
     )->cdata($title);
 
+    my ( $xe, $ye ) = &get_pos( 0, 3700, $bp_length );
+
+    if(0){
+    my ( $xe2, $ye2 ) = &get_pos( 0, 2850, $bp_length );
+    $oriter->path( 'd' => "M$xe2,$ye2 L$xe,$ye" );
+
+    ( $xe, $ye ) = &get_pos( 0, 2200, $bp_length );
+    ( $xe2, $ye2 ) = &get_pos( 0, 1300, $bp_length );
+    $oriter->path( 'd' => "M$xe2,$ye2 L$xe,$ye" );
+
+    my ( $xe, $ye ) = &get_pos( length($gb->{SEQ}), 3700, $bp_length );
+    my ( $xe2, $ye2 ) = &get_pos( length($gb->{SEQ}), 2850, $bp_length );
+    $oriter->path( 'd' => "M$xe2,$ye2 L$xe,$ye" );
+
+    ( $xe, $ye ) = &get_pos( length($gb->{SEQ}), 2200, $bp_length );
+    ( $xe2, $ye2 ) = &get_pos( length($gb->{SEQ}), 1300, $bp_length );
+    $oriter->path( 'd' => "M$xe2,$ye2 L$xe,$ye" );
+}
+
     open( PWRITER, ">./data/" . $wfile . ".coord" );
 
-    my ( $xe, $ye ) = &get_pos( $ori, 4000, $bp_length );
-    $oriter->path( 'd' => "M$cx,$cy L$xe,$ye" );
+    if($gb->{LOCUS}->{circular} == 1){
 
-    if ( $xe < $cx ) {
-        $cdst->text(
-            'text-anchor' => 'end',
-            x             => $xe,
-            y             => $ye,
-            'font-size'   => 100
-        )->cdata("ORIGIN");
-    }
-    else {
-        $cdst->text( x => $xe, y => $ye, 'font-size' => 100 )->cdata("ORIGIN");
-    }
+	( $xe, $ye ) = &get_pos( $ori, 4000, $bp_length );
+	$oriter->path( 'd' => "M$cx,$cy L$xe,$ye" );
+	
+	if ( $xe < $cx ) {
+	    $cdst->text(
+			'text-anchor' => 'end',
+			x             => $xe,
+			y             => $ye,
+			'font-size'   => 100
+			)->cdata("ORIGIN");
+	}
+	else {
+	    $cdst->text( x => $xe, y => $ye, 'font-size' => 100 )->cdata("ORIGIN");
+	}
+	
+	($xe, $ye ) = &get_pos( $ter, 4000, $bp_length );
+	$oriter->path( 'd' => "M$cx,$cy L$xe,$ye" );
 
-    ($xe, $ye ) = &get_pos( $ter, 4000, $bp_length );
-    $oriter->path( 'd' => "M$cx,$cy L$xe,$ye" );
-
-    if ( $xe < $cx ) {
-        $cdst->text(
-            'text-anchor' => 'end',
-            x             => $xe,
-            y             => $ye,
-            'font-size'   => 100
-        )->cdata("TERMINUS");
-    } else {
-        $cdst->text( x => $xe, y => $ye, 'font-size' => 100 )->cdata("TERMINUS");
+	if ( $xe < $cx ) {
+	    $cdst->text(
+			'text-anchor' => 'end',
+			x             => $xe,
+			y             => $ye,
+			'font-size'   => 100
+			)->cdata("TERMINUS");
+	} else {
+	    $cdst->text( x => $xe, y => $ye, 'font-size' => 100 )->cdata("TERMINUS");
+	}
+	
     }
 
     my $num = $bp_length;
     while ( $num =~ s/(.*\d)(\d\d\d)/$1,$2/ ) { }
 
+    my $text2 = $svg->text(
+        id            => 'chromosome',
+        'text-anchor' => 'middle',
+        x             => $cx,
+        y             => $cy + 200,
+        fill          => '#cccccc',
+        'font-size'   => 150,
+        'font-family' => 'Gill Sans'
+    )->cdata( sprintf("%s: %s", $gb->id(), $optionalText) ) if(length($optionalText));
+
     my $text2 = $svg->text(
         id            => 'bplength',
         'text-anchor' => 'middle',
@@ -393,52 +477,115 @@ sub circular_map{
     }
 
     for my $feature ( $gb->feature() ) {
+	my($xs, $ys, $xe, $ye);
+	my $strokeW = 300;
+	my $r = 3400;
+
         if ( $gb->{$feature}->{type} eq "gene" and $gb->{$feature}->{direction} eq "direct" ) {
-            $count_a++;
-            my $r = 3400;
-            my ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
-            my ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+            $r = 3400;
+            ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+            ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
 
-            if ( $count_a % 2 == 1 ) {
-                $cdsa->path(
-                    id  => "a" . $count_a,
-                    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye"
-                );
-            } else {
-                $cdsb->path(
-                    id  => "a" . $count_a,
-                    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye"
-                );
+            if ($gb->{LOCUS}->{circular} == 0){
+
+		if($gb->{$feature}->{start}/(length($gb->{SEQ})) > 47/48){
+		    my $startbp = $gb->{$feature}->{start} - int(length($gb->{SEQ}) * 47/48);
+		    my $startpx = $startbp / int(length($gb->{SEQ}) * 1/48);
+		    $strokeW = 300 - ($startpx * 300);
+		    $r += int($strokeW/2) - 150;
+		    ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+		    ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end},   $r, $bp_length );
+		}
             }
 
-            my ( $xm, $ym ) = &get_mapp( ( $xs + $xe ) / 2, ( $ys + $ye ) / 2, $bp_length );
-	    my $feat = $gb->next_feature($feature);
-            printf PWRITER "%s\t%s\t%s\t%d\t%d\n", $feat, $gb->{$feat}->{type}, $gb->{$feat}->{gene}, $xm, $ym;
+        }elsif ( $gb->{$feature}->{type} eq "gene" and $gb->{$feature}->{direction} eq "complement" ){
+            $r = 3100;
+            ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+            ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+
+            if ($gb->{LOCUS}->{circular} == 0){
+
+		if($gb->{$feature}->{start}/(length($gb->{SEQ})) > 47/48){
+		    my $startbp = $gb->{$feature}->{start} - int(length($gb->{SEQ}) * 47/48);
+		    my $startpx = $startbp / int(length($gb->{SEQ}) * 1/48);
+		    $strokeW = 300 - ($startpx * 300);
+		    $r -= int($strokeW/2) - 150;
+		    ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+		    ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+		}
+	    }
         }
 
-        if ( $gb->{$feature}->{type} eq "gene" and $gb->{$feature}->{direction} eq "complement" ){
-            $count_b++;
-            my $r = 3100;
-            my ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
-            my ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
-
-            if ( $count_b % 2 == 1 ) {
-                $cdsx->path(
-                    id  => "b" . $count_b,
-                    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye"
-                );
-            } else {
-                $cdsy->path(
-                    id  => "b" . $count_b,
-                    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye"
-                );
-            }
-
-            my ( $xm, $ym ) = &get_mapp( ( $xs + $xe ) / 2, ( $ys + $ye ) / 2, $bp_length );
-
+	if($gb->{$feature}->{type} eq "gene"){
+	    if ($gb->{LOCUS}->{circular} == 1){
+		if(query_strand($gb, $feature) eq 'leading'){
+		    $count_a++;
+		    if($count_a % 2 == 1 ) {
+			$cdsa->path(
+				    id  => "a" . $count_a,
+				    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye",
+				    'stroke-width'=>$strokeW
+				    );
+		    }else{
+			$cdsb->path(
+				    id  => "a" . $count_a,
+				    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye",
+				    'stroke-width'=>$strokeW
+				    );
+		    }
+		}else{
+		    $count_b++;
+		    if($count_a % 2 == 1 ) {
+			$cdsx->path(
+				    id  => "b" . $count_b,
+				    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye",
+				    'stroke-width' => $strokeW
+				    );
+		    } else {
+			$cdsy->path(
+				    id  => "b" . $count_b,
+				    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye",
+				    'stroke-width' => $strokeW
+				    );
+		    }
+		}
+	    }else{
+		if($gb->{$feature}->{direction} eq 'direct'){
+		    $count_a++;
+		    if($count_a % 2 == 1 ) {
+			$cdsa->path(
+				    id  => "a" . $count_a,
+				    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye",
+				    'stroke-width'=>$strokeW
+				    );
+		    }else{
+			$cdsb->path(
+				    id  => "a" . $count_a,
+				    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye",
+				    'stroke-width'=>$strokeW
+				    );
+		    }
+		}else{
+		    $count_b++;
+		    if($count_a % 2 == 1 ) {
+			$cdsx->path(
+				    id  => "b" . $count_b,
+				    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye",
+				    'stroke-width' => $strokeW
+				    );
+		    } else {
+			$cdsy->path(
+				    id  => "b" . $count_b,
+				    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye",
+				    'stroke-width' => $strokeW
+				    );
+		    }
+		}
+	    }
+	    my ( $xm, $ym ) = &get_mapp( ( $xs + $xe ) / 2, ( $ys + $ye ) / 2, $bp_length );
 	    my $feat = $gb->next_feature($feature);
-            printf PWRITER "%s\t%s\t%s\t%d\t%d\n", $feat, $gb->{$feat}->{type}, $gb->{$feat}->{gene}, $xm, $ym;
-        }
+	    printf PWRITER "%s\t%s\t%s\t%d\t%d\n", $feat, $gb->{$feat}->{type}, $gb->{$feat}->{gene}, $xm, $ym;
+	}
 
         if ( $gb->{$feature}->{type} eq "tRNA" and $gb->{$feature}->{direction} eq "direct" ){
             my $r = 2600;
@@ -477,16 +624,33 @@ sub circular_map{
         }
     }
 
+    if($gb->{LOCUS}->{circular} == 0){
+	my $ye1 = $cy - 3000;
+	my $ye2 = $cy - 3500;
+	$oriter->path( 'd' => "M$cx,$ye1 L$cx,$ye2", opacity=>0.5 );
+	
+	my $r = 3550;
+	my ( $rx, $ry2 ) = &get_pos( int(length($gb->{SEQ}) * 47/48), $r, $bp_length );
+	$oriter->path( 'd' => "M$cx,$ry L$rx,$ry2", opacity=>0.2 );
+	
+	my $r = 2950;
+	my ( $rx, $ry2 ) = &get_pos( int(length($gb->{SEQ}) * 47/48), $r, $bp_length );
+	$oriter->path( 'd' => "M$cx,$ry L$rx,$ry2", opacity=>0.2 );
+    }
+
+
     my $bline  = 2000;
     my $blineg = 1500;
     my ( $px,  $py )  = &get_pos( 0, $bline,  $bp_length );
     my ( $pxg, $pyg ) = &get_pos( 0, $blineg, $bp_length );
+    my $pgcc  = 0.5;
+    my $pgcs  = 0.0;
 
     my $range = 1000;
     my $wsize = 2000;           #window_bp_size
     my $step  = 1000;
 
-    $tag   = $svg->circle(
+    $tag = $svg->circle(
         cx     => $cx,
         cy     => $cy,
         r      => $bline,
@@ -509,25 +673,63 @@ sub circular_map{
         my $c_a = $temp =~ tr/a/a/;
         my $c_t = $temp =~ tr/t/t/;
 
+	my $opacity = 1;
+	if($i > (length($gb->{SEQ}) * 95/96)){# || $i < length($gb->{SEQ}) * 1/96){
+	    $opacity = 0.1 if($gb->{LOCUS}->{circular} == 0);
+	}
+
         my $gc_content = 0;
         $gc_content = ( $c_g + $c_c ) / ( $c_g + $c_c + $c_a + $c_t )
           if ( $c_g + $c_c + $c_a + $c_t );
 
         my ( $x, $y ) = &get_pos( $i, ( $bline + ( $gc_content - 0.5 ) * $range * 2 ), $bp_length );
 
+	if($pgcc >= 0.5 && $gc_content >= 0.5){
+	    $gcska->path( 'd' => "M$px,$py L$x,$y", opacity=>$opacity );
+	}elsif($pgcc <= 0.5 && $gc_content <= 0.5){
+	    $gcskc->path( 'd' => "M$px,$py L$x,$y", opacity=>$opacity );
+	}else{
+	    my ( $xn, $yn ) = &get_pos( $i, $bline, $bp_length );
+	    if($gc_content > 0.5){
+		$gcska->path( 'd' => "M$xn,$yn L$x,$y", opacity=>$opacity );
+		$gcskc->path( 'd' => "M$px,$py L$xn,$yn", opacity=>$opacity );
+	    }else{
+		$gcska->path( 'd' => "M$px,$py L$xn,$yn", opacity=>$opacity );
+		$gcskc->path( 'd' => "M$xn,$yn L$x,$y", opacity=>$opacity );
+	    }
+	}
+
         my $gc_skew = 0;
         $gc_skew = ( $c_c - $c_g ) / ( $c_g + $c_c ) if ( $c_g + $c_c );
 
         my ( $xg, $yg ) = &get_pos( $i, ( $blineg + ($gc_skew) * $range * 2 ), $bp_length );
 
-        $gcska->path( 'd' => "M$px,$py L$x,$y" );
+	if($pgcs >= 0 && $gc_skew >= 0){
+	    $gcskb->path( 'd' => "M$pxg,$pyg L$xg,$yg", opacity=>$opacity );
+	}elsif($pgcs <= 0 && $gc_skew <= 0){
+	    $gcskd->path( 'd' => "M$pxg,$pyg L$xg,$yg", opacity=>$opacity );
+	}else{
+	    my ( $xn, $yn ) = &get_pos( $i, $blineg, $bp_length );
+	    if($gc_skew > 0){
+		$gcskb->path( 'd' => "M$xn,$yn L$xg,$yg",   opacity=>$opacity );
+		$gcskd->path( 'd' => "M$pxg,$pyg L$xn,$yn", opacity=>$opacity );
+	    }else{
+		$gcskb->path( 'd' => "M$pxg,$pyg L$xn,$yn", opacity=>$opacity );
+		$gcskd->path( 'd' => "M$xn,$yn L$xg,$yg",   opacity=>$opacity );
+	    }
+	}
+
+#        $gcskb->path( 'd' => "M$pxg,$pyg L$xg,$yg", opacity=>$opacity );
+
 
         $px = $x;
         $py = $y;
-        $gcskb->path( 'd' => "M$pxg,$pyg L$xg,$yg" );
 
         $pxg = $xg;
         $pyg = $yg;
+
+	$pgcc = $gc_content;
+	$pgcs = $gc_skew;
     }
 
     $cdst->text(
@@ -554,6 +756,21 @@ sub circular_map{
     }
 }
 
+
+sub dec2hex {
+    my $num = shift;
+    return $num if ($num < 10);
+
+    $num = 'a' if ($num == 10);
+    $num = 'b' if ($num == 11);
+    $num = 'c' if ($num == 12);
+    $num = 'd' if ($num == 13);
+    $num = 'e' if ($num == 14);
+    $num = 'f' if ($num == 15);
+
+    return $num;
+}
+    
 sub get_pos {
     my ( $n, $r, $bp_length ) = @_;
     my $x = $cx + ( sin( 2 * $pi * $n / $bp_length ) * $r );
@@ -1400,7 +1617,7 @@ sub genome_map2 {
     -width          image width (default: 8192)
     -height         image height (default: 8192)
     -output         "g" for graph, "show" for display (default: "show") 
-    -datafilename   coordinate file name (default: "data.coord")
+    -datafilename   coordinate file name (default: "[accessionNumber].coord")
     -gmap           1 to create Google Map view, 0 to only generate PNG image
                     (default:1)
 
@@ -1408,6 +1625,7 @@ sub genome_map2 {
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+   20100319-01 added support for exons
    20071008-01 added -gmap option
    20070628-01 added -datafilename option
    20070623-01 initial posting
@@ -1415,11 +1633,11 @@ sub genome_map2 {
 =cut
 
 sub genome_map3 {
-    &opt_default(output=>"show", width=>8192, height=>8192, datafilename=>"data.coord", gmap=>1);
+    &opt_default(output=>"show", width=>8192, height=>8192, datafilename=>'', gmap=>1);
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $acnum = $gb->{LOCUS}->{id};
-    my $datafilename = opt_val("datafilename");
+    my $datafilename = opt_val("datafilename") || $gb->id() . '.coord';
     my $output = opt_val("output");
     my $filename;
     my $topmargin = 50;
@@ -1464,7 +1682,6 @@ sub genome_map3 {
     my $ggreen  = $im->colorAllocate(180,220,180);   #T for graph
     my $gblue   = $im->colorAllocate(200,200,255);   #C for graph
   
-
     # Draw Base Graph
     for ($i = $sidemargin; $i <= $sidemargin + $hblock * $numHBlock; $i += $hblock){
 	$im->line($i, $topmargin, $i, $topmargin + 5 * ($numVBlock + 1) * $vblock, $gray);
@@ -1568,9 +1785,9 @@ sub genome_map3 {
     foreach my $feat ($gb->feature()){
 	next unless($gb->{$feat}->{type} eq 'CDS' || $gb->{$feat}->{type} eq 'rRNA' || $gb->{$feat}->{type} eq 'tRNA');
 	my $cdsstart = $gb->{$feat}->{start};
-	my $cdsend = $gb->{$feat}->{end};
-	my $cdsdir = $gb->{$feat}->{direction};
-	my $cdsdiff = $cdsstart - $before;
+	my $cdsend   = $gb->{$feat}->{end};
+	my $cdsdir   = $gb->{$feat}->{direction};
+	my $cdsdiff  = $cdsstart - $before;
 	my $cdsdiff2 = $cdsstart - $before2;
 	my $genename = $gb->{$feat}->{gene};
 
@@ -1603,11 +1820,13 @@ sub genome_map3 {
 
 	unless ($cdsstart >= $end){
 	    
-	    my $dif1 = -2; 
+	    my $dif1 = -2;
 	    my $dif2 = -7 - 2;
+	    my $dif3 = $dif2 - 5;
 	    if ($cdsdir eq 'complement'){
 		$dif1 *= -1;
 		$dif2 *= -1;
+		$dif3 *= -1;
 	    }
 
 	    if($cdsdir eq 'complement'){
@@ -1622,19 +1841,63 @@ sub genome_map3 {
 		$topmargin + $dif1 + (int(($cdsend / $window) / ($hblock * $numHBlock))+1) * $vblock * 5;
 	    }
 
-
 	    my $k;
-	    for ($k = $cdsstart-$start; $k <= $cdsend - $start; $k += $window){
+	    if($gb->{$feat}->{join}){
+		my $prev = 0;
+		for my $exon (split(/,/, $gb->{$feat}->{join})){
+		    my ($exonstart, $exonend) = split(/\.\./, $exon, 2);
+		    for ($k = $exonstart-$start; $k <= $exonend - $start; $k += $window){
+			last if ($k + $start > $end);
+			my $l = int ($k / $window);
+			$im->line($sidemargin + 1 + $l % ($hblock * $numHBlock),
+				  $topmargin + $dif1 + (int($l/($hblock * $numHBlock))+1)*$vblock * 5,
+				  $sidemargin + 1 + $l % ($hblock * $numHBlock),
+				  $topmargin + $dif2 + (int($l/($hblock * $numHBlock))+1)*$vblock * 5,
+				  $blockcolor);
+		    }
+		    unless($prev == 0){
+			my $l1 = int (($prev - $start) / $window);
+			my $l2 = int (($exonstart - $start) / $window);
+			my $l3 = $l1 + int(($l2 - $l1)/2);
+			if((int($l1/($hblock * $numHBlock))+1)*$vblock * 5 != (int($l2/($hblock * $numHBlock))+1)*$vblock * 5){
+			    $im->line($sidemargin + 1 + $l1 % ($hblock * $numHBlock),
+				      $topmargin + $dif2 + (int($l1/($hblock * $numHBlock))+1)*$vblock * 5,
+				      $sidemargin + 1 + $hblock * $numHBlock,
+				      $topmargin + $dif3 + (int($l1/($hblock * $numHBlock))+1)*$vblock * 5,
+				      $black);
+			    $im->line($sidemargin + 1,
+				      $topmargin + $dif3 + (int($l2/($hblock * $numHBlock))+1)*$vblock * 5,
+				      $sidemargin + 1 + $l2 % ($hblock * $numHBlock),
+				      $topmargin + $dif2 + (int($l2/($hblock * $numHBlock))+1)*$vblock * 5,
+				      $black);
+			}else{
+			    $im->line($sidemargin + 1 + $l1 % ($hblock * $numHBlock),
+				      $topmargin + $dif2 + (int($l3/($hblock * $numHBlock))+1)*$vblock * 5,
+				      $sidemargin + 1 + $l3 % ($hblock * $numHBlock),
+				      $topmargin + $dif3 + (int($l3/($hblock * $numHBlock))+1)*$vblock * 5,
+				      $black);
+			    $im->line($sidemargin + 1 + $l2 % ($hblock * $numHBlock),
+				      $topmargin + $dif2 + (int($l3/($hblock * $numHBlock))+1)*$vblock * 5,
+				      $sidemargin + 1 + $l3 % ($hblock * $numHBlock),
+				      $topmargin + $dif3 + (int($l3/($hblock * $numHBlock))+1)*$vblock * 5,
+				      $black);
+			}
+		    }
+
+		    $prev = $exonend;
+		}
+	    }else{
+		for ($k = $cdsstart-$start; $k <= $cdsend - $start; $k += $window){
+		    last if ($k + $start > $end);
+		    my $l = int ($k / $window);
+		    $im->line($sidemargin + 1 + $l % ($hblock * $numHBlock),
+			      $topmargin + $dif1 + (int($l/($hblock * $numHBlock))+1)*$vblock * 5,
+			      $sidemargin + 1 + $l % ($hblock * $numHBlock),
+			      $topmargin + $dif2 + (int($l/($hblock * $numHBlock))+1)*$vblock * 5,
+			      $blockcolor);
+		}
 		last if ($k + $start > $end);
-		my $l = int ($k / $window);
-		$im->line($sidemargin + 1 + $l % ($hblock * $numHBlock),
-			  $topmargin + $dif1 + (int($l/($hblock * $numHBlock))+1)*$vblock * 5,
-			  $sidemargin + 1 + $l % ($hblock * $numHBlock),
-			  $topmargin + $dif2 + (int($l/($hblock * $numHBlock))+1)*$vblock * 5,
-			  $blockcolor);
 	    }
-	    last if ($k + $start > $end);
-	    
 	    $k = int(($cdsstart - $start)/$window);
 	    
 	    $dif1 = -2; 
