@@ -376,12 +376,12 @@ sub to_fastq{
     1. Fisher R.A. and Yates F. (1938) "Example 12", Statistical Tables, London
     2. Durstenfeld R. (1964) "Algorithm 235: Random permutation", CACM 7(7):420
     3. Jiang M., Anderson J., Gillespie J., and Mayne M. (2008) "uShuffle: 
-       a useful ool for shuffling biological sequences while preserving the k-let
+       a useful tool for shuffling biological sequences while preserving the k-let
        counts", BMC Bioinformatics 9:192
     4. Kandel D., Matias Y., Unver R., and Winker P. (1996) "Shuffling biological 
        sequences", Discrete Applied Mathematics 71(1-3):171-185  
     5. Propp J.G. and Wilson D.B. (1998) "How to get a perfectly random sample 
-       "from a generic Markov chain and generate a random spanning tree of a 
+       from a generic Markov chain and generate a random spanning tree of a 
        directed graph", Journal of Algorithms 27(2):170-217
 
   Author:
