@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Util.pm,v 1.1.1.1 2002/04/02 20:25:43 gaou Exp $
@@ -42,13 +42,14 @@ use autouse 'Cwd'=>qw(getcwd);
 use SelfLoader;
 
 our @EXPORT = qw(
-		 find_king_of_gene
 		 seqinfo
 		 longest_ORF
 		 molecular_weight
 		 view_cds
+		 generate_oligomers
 		 oligomer_translation
 		 filter_cds_by_atg
+		 find_king_of_gene
 		 );
 
 
@@ -74,8 +75,46 @@ __DATA__
 #        Methods Start
 #::::::::::::::::::::::::::::::
 
+=head2 generate_oligomers
 
+  Name: generate_oligomers   -   generate all oligomers of given length
 
+  Description:
+    This method retuns a list of all oligomers of given length. 
+    For example, for length 2, this method returns a list of all 
+    di-nucleotides (aa,at,ag,ac,ta,tt,tg,tc,ga,gt,gg,gc,ca,ct,cg,cc),
+    and for length 8, returns all 65536 octamers.
+
+ Usage: 
+    @oligomers = generate_oligomers($length)
+
+ Options:
+    None.
+
+  Author: Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20100114-01 initial posting
+
+=cut
+
+sub generate_oligomers{
+    my $num = shift || 8;
+
+    local *transferase = sub{
+	my @result;
+	foreach my $key (@_){
+	    push(@result, $key . $_) for (qw/a t g c/);
+	}
+	return @result;
+    };
+
+    my @result = ('');
+    for (1..$num){
+	@result = &transferase(@result);
+    }
+    return @result;
+}
 
 
 =head2 seqinfo
@@ -85,17 +124,20 @@ __DATA__
   Description:
     This method prints out basic compositional statistics of the 
     given nucleotide sequence, in a format similar to the one printed
-    right after calling load().
+    right after calling load(). Returns the number of A, T, G, and C bases.
 
  Usage: 
-    seqinfo($genome)
+    ($a, $t, $g, $c) = seqinfo($genome)
 
  Options:
-   none
+    Supply something as the second argument to suppress standard output.
+    ex. ($a, $t, $g, $c) = seqinfo($genome, 1)
+    This is useful for simple counting of the four nucleotides.
 
   Author: Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20100108-01 added the second argument suppresing output.
     20080428-01 integrated into G::IO::Handler::seq_info()
     20020207-01 initial posting
 
@@ -103,7 +145,7 @@ __DATA__
 
 
 sub seqinfo {
-    return opt_as_gb(shift @_)->seq_info();
+    return opt_as_gb(shift @_)->seq_info(shift @_);
 }
 
 
