@@ -30,6 +30,8 @@ package G::Shell::Help;
 use strict;
 use base qw(Exporter);
 use autouse 'String::Approx'=>qw(amatch);
+
+use Carp;
 use File::Find;
 use G::DB::SDB;
 use G::Messenger;
@@ -130,6 +132,8 @@ sub buildHelpDB{
    -w         search through BioMOBY web services
               when the <keyword> is the name of available service,
               description of the web service is provided.
+              To use this option, you must first install MOBY::Client::Central
+              module from CPAN. 
 
   Author: 
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
@@ -154,7 +158,12 @@ sub help{
 	my $ans = "     Keyword \"$key\" was not found in the documentations.\n";
 
 	if($option eq 'w'){
-	    require MOBY::Client::Central;
+	    eval{
+		require MOBY::Client::Central;
+	    };
+	    if($@){
+		croak "MOBY::Client::Central does not seem to be installed. Try 'cpan MOBY::Client::Central' to install this module to enable BioMOBY support.";
+	    }
 	    my $m = MOBY::Client::Central->new();
 
 	    my ($sv, undef) = $m->findService(serviceName=>$key);
