@@ -119,7 +119,7 @@ __DATA__
 
 
 sub generateGMap{
-    opt_default(level=>6, posterize=>16, colors=>255, cgr=>0, apikey=>"YOUR_GOOGLE_MAP_KEY_HERE");
+    opt_default(level=>6, posterize=>16, colors=>255, cgr=>0, apikey=>"YOUR_GOOGLE_MAP_KEY_HERE", vips=>1);
     my @argv       = opt_get(@_);
     my $file       = shift @argv;
     my $level      = opt_val("level");
@@ -127,8 +127,12 @@ sub generateGMap{
     my $posterize  = opt_val("colors");
     my $cgr        = opt_val("cgr");
     my $apikey     = opt_val("apikey");
+    my $optvips    = opt_val("vips");
     $posterize = 16 if($cgr);
 
+    my $vips    = $optvips ? `whereis vips` : '';
+    $vips       =~ s/vips:\s+//g;
+    my $tmpname = '/tmp/' . time(); 
     $level = 3 if ($level < 3);
 
     my $filename = basename($file);
@@ -140,14 +144,8 @@ sub generateGMap{
 
     mkdir($name);
 
-    my ($width, $height);
-    if(`identify $file` =~ /\s(\d+)x(\d+)\s/){
-	$width = $1;
-	$height = $2;
-    }else{
-	die("Either ImageMagick is not available, or the given file is not a proper image.");
-    }
-    
+    require Image::Size;
+    my ($width, $height) = Image::Size::imgsize($file);
     my $maxwidth = 256 * 2 ** ($level - 1);
     
     if($width == $height){
@@ -174,32 +172,71 @@ sub generateGMap{
 
     my $maxlevel = $level - 1;
 
-    for(my $i = $maxlevel; $i >= 0; $i --){
-	my $tiles = 2 ** $i;
-	my $size = $tiles * 256;
-
-	my $pid = fork();
-
-	if($pid){
-	}elsif(defined $pid){
+    my $pid = fork();
+    
+    if($pid){
+    }elsif(defined $pid){
+	for(my $i = $maxlevel; $i >= 1; $i --){
+	    my $tiles = 2 ** $i;
+	    my $size = $tiles * 256;
 	    my $j = $i - 1;
-	    exit if ($j < 0);
-	    system(sprintf("convert +dither -colors $posterize level%d.png -size %dx%d -resize %dx%d level%d.png", $i, $size, $size, $size/2, $size/2, $j)) unless(-e "level$j.png");
-	    exit;
-	}else{
-	    die("fork failed.");
-	}
 
-	system("convert +dither -colors $posterize level$i.png -crop 256x256 level$i-%d.png");
+#	    msg_error "shrink level$j\n";
 
-	for(my $y = 1; $y <= $tiles; $y ++){
-	    for(my $x = 1; $x <= $tiles; $x ++){
-		rename(sprintf("level%d-%d.png", $i, ($y - 1) * $tiles + ($x - 1)), sprintf("level%d-%d-%d.png", $i, $x - 1, $y - 1));
+	    if($vips){
+		if($j == 0){
+		    system(sprintf("vips im_shrink level%d.png level0-0-0.png 2 2", $i, $j)) unless(-e "level$j.png");
+		}else{
+		    system(sprintf("vips im_shrink level%d.png level%d.png 2 2", $i, $j)) unless(-e "level$j.png");
+		}
+	    }else{
+		if($j == 0){
+		    system(sprintf("convert +dither -colors $posterize level%d.png -size %dx%d -resize %dx%d level0-0-0.png", $i, $size, $size, $size/2, $size/2)) unless(-e "level$j.png");
+		}else{
+		    system(sprintf("convert +dither -colors $posterize level%d.png -size %dx%d -resize %dx%d level%d.png", $i, $size, $size, $size/2, $size/2, $j)) unless(-e "level$j.png");
+		}
 	    }
 	}
-
-	waitpid($pid, 0);
+	exit;
+    }else{
+	die("fork failed.");
     }
+
+    for(my $i = $maxlevel; $i >= 1; $i --){
+	my $tiles = 2 ** $i;
+	my $size  = $tiles * 256;
+
+	my $checkfile = 'level0-0-0.png';
+	$checkfile = 'level' . ($i - 1) . '.png' unless($i == 1);
+
+#	msg_error("checking availability of $checkfile\n");
+
+	unless(-e $checkfile || $i == $maxlevel){
+	    sleep(1);
+	    redo();
+	}
+
+#	msg_error "start generation of level $i\n";
+
+	if($vips){
+	    for my $c (1..$tiles){
+		$c --;
+
+		system("vips im_extract_area level$i.png $tmpname-level$i-$c.png " . $c * 256 . " 0 256 $size &&" .
+		       "convert +dither -colors $posterize $tmpname-level$i-$c.png -crop 256x256 level$i-$c-%d.png &&" .
+		       "rm $tmpname-level$i-$c.png $and");
+	    }
+	}else{
+	    system("convert +dither -colors $posterize level$i.png -crop 256x256 level$i-%d.png");
+	    
+	    for(my $y = 1; $y <= $tiles; $y ++){
+		for(my $x = 1; $x <= $tiles; $x ++){
+		    rename(sprintf("level%d-%d.png", $i, ($y - 1) * $tiles + ($x - 1)), sprintf("level%d-%d-%d.png", $i, $x - 1, $y - 1));
+		}
+	    }
+	}
+    }
+    waitpid($pid, 0);
  
     open(OUT, '>' . 'index.html') || die($!);
     
