@@ -199,6 +199,9 @@ sub togoWS{
     This is a wrapper around MOBY::Client::Service to run web services
     through BioMOBY. Use help -w <keyword> to search for web services,
     help -w <service name> to view service description. 
+
+    To use this function, you must first install MOBY::Client::Central
+    module from CPAN. 
     
  Usage:
     ws(<service_name>, $argument1=>$ontology_or_namespace, ...);
@@ -231,8 +234,14 @@ sub togoWS{
 sub ws {
     my $service = shift;
 
-    require MOBY::Client::Central;
-    require MOBY::Client::Service;
+    eval{
+	require MOBY::Client::Central;
+	require MOBY::Client::Service;
+    };
+    if($@){
+	croak "MOBY::Client::Central does not seem to be installed. Try'cpan MOBY::Client::Central' to install this module to enable BioMOBY support.";
+    }
+
     my $m = MOBY::Client::Central->new();
     my %ns = %{$m->retrieveNamespaces()};
 
