@@ -154,7 +154,7 @@ sub exec{
     close($fh);
 
     system("/usr/bin/env R " . $this->{save} . " --slave < "
-	   . $this->{cmd} . " >& " . $this->{log});
+	   . $this->{cmd} . " > " . $this->{log} . ' 2>&1');
 
     open(my $fh2, $this->{log});
     while(<$fh2>){
@@ -177,7 +177,12 @@ sub exec{
 		last;
 	    }
 	    warn("Warning in R...\n");
+	}elsif(/^Read \d+ items/){
+
+	}else{
+	    print STDERR $_;
 	}
+
     }
     close($fh2);
 
