@@ -99,7 +99,7 @@ sub opt_as_gb {
     } elsif (ref $ref eq 'SCALAR'){
 	if(-e $unknown){
 	    return new G::IO($unknown, 'no msg');
-	}elsif(length($unknown) < 25){
+	}elsif(length($unknown) < 25 && $unknown =~ /[^a-zA-Z\/]/){
 	    return new G::IO($unknown, 'no msg');
 	}else{
 	    my $new_gb = new G::IO('blessed');
